@@ -701,6 +701,18 @@ int rpc_lsarpc_write_request(struct ksmbd_rpc_pipe *pipe)
 	return lsarpc_invoke(pipe);
 }
 
+static void free_ph_entry(gpointer k, gpointer s, gpointer user_data)
+{
+	g_free(s);
+}
+
+static void lsarpc_ph_clear_table(void)
+{
+	g_rw_lock_writer_lock(&ph_table_lock);
+	g_hash_table_foreach(ph_table, free_ph_entry, NULL);
+	g_rw_lock_writer_unlock(&ph_table_lock);
+}
+
 int rpc_lsarpc_init(void)
 {
 	char domain_string[NAME_MAX];
@@ -724,6 +736,7 @@ void rpc_lsarpc_destroy(void)
 {
 	g_free(domain_name);
 	if (ph_table) {
+		lsarpc_ph_clear_table();
 		g_hash_table_destroy(ph_table);
 		ph_table = NULL;
 	}
